@@ -398,7 +398,7 @@ tree_open_pkg(tree_cat_ctx *cat_ctx, const char *name)
 	/* see if this pkg matches the query, here we can finally check
 	 * version conditions like >=, etc. */
 	if (cat_ctx->ctx->query_atom != NULL) {
-		(void)tree_get_atom(pkg_ctx, false);
+		(void)tree_get_atom(pkg_ctx, cat_ctx->ctx->query_atom->SLOT != NULL);
 		if (atom_compare(pkg_ctx->atom, cat_ctx->ctx->query_atom) != EQUAL) {
 			tree_close_pkg(pkg_ctx);
 			return NULL;
@@ -621,7 +621,7 @@ tree_read_file_pms(tree_pkg_ctx *pkg_ctx)
 		goto err;
 
 	len = sizeof(*ret) + s.st_size + 1;
-	ret = xmalloc(len);
+	ret = xzalloc(len);
 	ptr = (char*)ret + sizeof(*ret);
 	if ((off_t)fread(ptr, 1, s.st_size, f) != s.st_size)
 		goto err;
@@ -690,7 +690,8 @@ tree_read_file_md5(tree_pkg_ctx *pkg_ctx)
 
 	len = sizeof(*ret) + s.st_size + 1;
 	ret = xmalloc(len);
-	ptr = (char*)ret + sizeof(*ret);
+	memset(ret, 0, sizeof(*ret));
+	ptr = (char *)ret + sizeof(*ret);
 	if ((off_t)fread(ptr, 1, s.st_size, f) != s.st_size)
 		goto err;
 	ptr[s.st_size] = '\0';
@@ -803,6 +804,7 @@ tree_read_file_ebuild(tree_pkg_ctx *pkg_ctx)
 
 	len = sizeof(*ret) + s.st_size + 1;
 	ret = xmalloc(len);
+	memset(ret, 0, sizeof(*ret));
 	p = (char *)ret + sizeof(*ret);
 	if ((off_t)fread(p, 1, s.st_size, f) != s.st_size)
 		goto err;
