@@ -220,7 +220,7 @@ makeargv(const char *string, int *argc, char ***argv)
 {
 	int curc = 2;
 	char *q, *p, *str;
-	(*argv) = xmalloc(sizeof(char **) * curc);
+	(*argv) = xmalloc(sizeof(char *) * curc);
 
 	*argc = 1;
 	(*argv)[0] = xstrdup(argv0);
@@ -243,7 +243,7 @@ makeargv(const char *string, int *argc, char ***argv)
 
 		if (*argc == curc) {
 			curc *= 2;
-			(*argv) = xrealloc(*argv, sizeof(char **) * curc);
+			(*argv) = xrealloc(*argv, sizeof(char *) * curc);
 		}
 		(*argv)[*argc] = xstrdup(str);
 		(*argc)++;
@@ -667,6 +667,7 @@ read_one_repos_conf(const char *repos_conf, char **primary)
 			} else if (*q == '#') {
 				do_trim = true;
 				*q = '\0';
+				e = NULL;
 				r = q - 1;
 			} else {
 				if (*q == '=')
