@@ -159,7 +159,8 @@ list_dir(char ***retlist, size_t *retcnt, const char *path)
 		}
 		closedir(d);
 
-		qsort(rlist, rlen, sizeof(rlist[0]), compare_strings);
+		if (rlen > 1)
+			qsort(rlist, rlen, sizeof(rlist[0]), compare_strings);
 
 		*retlist = rlist;
 		*retcnt = rlen;
@@ -220,8 +221,8 @@ write_hashes(
 
 	if (m != NULL)
 		fwrite(data, len, 1, m);
-	if (gm != NULL)
-		gzwrite(gm, data, len);
+	if (gm != NULL && gzwrite(gm, data, len) == 0)
+		fprintf(stderr, "failed to write to compressed stream\n");
 }
 
 /**
@@ -365,7 +366,6 @@ parse_layout_conf(const char *path)
 			last_nl++;  /* skip \n */
 			len = last_nl - buf;
 			memmove(buf, last_nl, len);
-			last_nl = buf;
 		} else {
 			/* skip too long line */
 			len = 0;
@@ -431,7 +431,12 @@ generate_dir(const char *dir, enum type_manifest mtype)
 				"IGNORE lost+found\n"
 				"IGNORE packages\n"
 				"IGNORE snapshots\n");
-		gzwrite(mf, path, len);
+		if (gzwrite(mf, path, len) == 0) {
+			fprintf(stderr, "failed to write to file '%s/%s': %s\n",
+					dir, str_manifest_files_gz, strerror(errno));
+			gzclose(mf);
+			return NULL;
+		}
 
 		if (list_dir(&dentries, &dentrieslen, dir) != 0)
 			return NULL;
@@ -467,6 +472,9 @@ generate_dir(const char *dir, enum type_manifest mtype)
 						fprintf(stderr, "generating Manifest for %s failed!\n",
 								path);
 						gzclose(mf);
+						for (; i < dentrieslen; i++)
+							free(dentries[i]);
+						free(dentries);
 						return NULL;
 					}
 
@@ -531,11 +539,18 @@ generate_dir(const char *dir, enum type_manifest mtype)
 					"IGNORE timestamp.chk\n"
 					"IGNORE timestamp.commit\n"
 					"IGNORE timestamp.x\n");
-			gzwrite(mf, path, len);
+			if (gzwrite(mf, path, len) == 0) {
+				fprintf(stderr, "failed to write to file '%s/%s': %s\n",
+						dir, str_manifest_gz, strerror(errno));
+				gzclose(mf);
+				return NULL;
+			}
 		}
 
-		if (list_dir(&dentries, &dentrieslen, dir) != 0)
+		if (list_dir(&dentries, &dentrieslen, dir) != 0) {
+			gzclose(mf);
 			return NULL;
+		}
 
 		for (i = 0; i < dentrieslen; i++) {
 			/* ignore existing Manifests */
@@ -563,6 +578,7 @@ generate_dir(const char *dir, enum type_manifest mtype)
 	} else if (mtype == CATEGORY_MANIFEST) {
 		const char *mfest;
 		gzFile mf;
+		const char *ret = str_manifest_gz;
 
 		snprintf(path, sizeof(path), "%s/%s", dir, str_manifest_gz);
 		if ((mf = gzopen(path, "wb9")) == NULL) {
@@ -571,8 +587,10 @@ generate_dir(const char *dir, enum type_manifest mtype)
 			return NULL;
 		}
 
-		if (list_dir(&dentries, &dentrieslen, dir) != 0)
+		if (list_dir(&dentries, &dentrieslen, dir) != 0) {
+			gzclose(mf);
 			return NULL;
+		}
 
 		for (i = 0; i < dentrieslen; i++) {
 			/* ignore existing Manifests */
@@ -589,13 +607,13 @@ generate_dir(const char *dir, enum type_manifest mtype)
 					if (mfest == NULL) {
 						fprintf(stderr, "generating Manifest for %s failed!\n",
 								path);
-						gzclose(mf);
-						return NULL;
+						tv[0].tv_sec = 0;
+						ret = NULL;
+					} else {
+						snprintf(path, sizeof(path), "%s/%s",
+								dentries[i], mfest);
+						write_hashes(tv, dir, path, "MANIFEST", NULL, mf);
 					}
-
-					snprintf(path, sizeof(path), "%s/%s",
-							dentries[i], mfest);
-					write_hashes(tv, dir, path, "MANIFEST", NULL, mf);
 				} else if (s.st_mode & S_IFREG) {
 					write_hashes(tv, dir, dentries[i], "DATA", NULL, mf);
 				} /* ignore other "things" (like symlinks) as they
@@ -617,7 +635,7 @@ generate_dir(const char *dir, enum type_manifest mtype)
 			utimes(dir, tv);
 		}
 
-		return str_manifest_gz;
+		return ret;
 	} else if (mtype == EBUILD_MANIFEST) {
 		char newmanifest[8192];
 		FILE *m;
@@ -679,7 +697,11 @@ generate_dir(const char *dir, enum type_manifest mtype)
 		fclose(m);
 
 		snprintf(path, sizeof(path), "%s/%s", dir, str_manifest);
-		rename(newmanifest, path);
+		if (rename(newmanifest, path) == -1) {
+			fprintf(stderr, "failed to rename file '%s' to '%s': %s\n",
+					newmanifest, path, strerror(errno));
+			return NULL;
+		}
 
 		if (tv[0].tv_sec != 0) {
 			/* set Manifest and dir mtime to most recent file we found */
@@ -1478,7 +1500,8 @@ verify_manifest(
 	 * - Manifest at top-level, needs to be igored as it only points to
 	 *   the larger Manifest.files.gz
 	 */
-	qsort(elems, elemslen, sizeof(elems[0]), compare_elems);
+	if (elemslen > 1)
+		qsort(elems, elemslen, sizeof(elems[0]), compare_elems);
 	snprintf(buf, sizeof(buf), "%s/%s", dir, manifest);
 	ret = verify_dir(dir, elems, elemslen, 0, buf + 2, msgs);
 	checked_manifests++;
@@ -1762,8 +1785,10 @@ qmanifest_main(int argc, char **argv)
 		}
 	}
 
-	if ((curdirfd = open(".", O_RDONLY)) < 0)
+	if ((curdirfd = open(".", O_RDONLY)) < 0) {
 		warn("cannot open current directory?!? %s\n", strerror(errno));
+		return EXIT_FAILURE;
+	}
 
 	ret = EXIT_SUCCESS;
 	argc -= optind;
