@@ -767,7 +767,8 @@ merge_tree_at(int fd_src, const char *src, int fd_dst, const char *dst,
 		return ret;
 	}
 
-	dir = fdopendir(subfd_src);
+	i = dup(subfd_src);  /* fdopendir closes its argument */
+	dir = fdopendir(i);
 	if (!dir)
 		goto done;
 
@@ -1069,6 +1070,7 @@ pkg_merge(int level, const depend_atom *atom, const struct pkg_t *pkg)
 						if (strlen(resolved) < 1) {
 							warn("Cant find a binpkg for %s from rdepend(%s)",
 									name, pkg->RDEPEND);
+							atom_implode(subatom);
 							continue;
 						}
 
@@ -1601,10 +1603,16 @@ static int
 unlink_empty(const char *buf)
 {
 	struct stat st;
-	if (stat(buf, &st) != -1)
+	int fd;
+	int ret = -1;
+
+	fd = open(buf, O_RDONLY);
+	if (fd != -1 && stat(buf, &st) != -1) {
 		if (st.st_size == 0)
-			return unlink(buf);
-	return -1;
+			ret = unlink(buf);
+		close(fd);
+	}
+	return ret;
 }
 
 static int
@@ -1669,7 +1677,10 @@ pkg_fetch(int level, const depend_atom *atom, const struct pkg_t *pkg)
 	unlink_empty(buf);
 
 	snprintf(str, sizeof(str), "%s/%s", pkgdir, pkg->CATEGORY);
-	mkdir(str, 0755);
+	if (mkdir(str, 0755) == -1 && errno != EEXIST) {
+		warn("Failed to create %s", str);
+		return;
+	}
 
 	/* XXX: should do a size check here for partial downloads */
 
@@ -1701,7 +1712,8 @@ pkg_fetch(int level, const depend_atom *atom, const struct pkg_t *pkg)
 		snprintf(buf, sizeof(buf), "%s/%s.tbz2", atom->CATEGORY, pkg->PF);
 		fetch(str, buf);
 	}
-	snprintf(buf, sizeof(buf), "%s/%s/%s.tbz2", pkgdir, atom->CATEGORY, pkg->PF);
+	snprintf(buf, sizeof(buf), "%s/%s/%s.tbz2",
+			pkgdir, atom->CATEGORY, pkg->PF);
 	if (access(buf, R_OK) != 0) {
 		snprintf(buf, sizeof(buf), "%s.tbz2", pkg->PF);
 		fetch(str, buf);
@@ -1709,7 +1721,8 @@ pkg_fetch(int level, const depend_atom *atom, const struct pkg_t *pkg)
 	}
 
 	/* verify the pkg exists now. unlink if zero bytes */
-	snprintf(buf, sizeof(buf), "%s/%s/%s.tbz2", pkgdir, atom->CATEGORY, pkg->PF);
+	snprintf(buf, sizeof(buf), "%s/%s/%s.tbz2",
+			pkgdir, atom->CATEGORY, pkg->PF);
 	unlink_empty(buf);
 
 	if (access(buf, R_OK) != 0) {
@@ -1718,7 +1731,8 @@ pkg_fetch(int level, const depend_atom *atom, const struct pkg_t *pkg)
 		return;
 	}
 
-	snprintf(buf, sizeof(buf), "%s/%s/%s.tbz2", pkgdir, atom->CATEGORY, pkg->PF);
+	snprintf(buf, sizeof(buf), "%s/%s/%s.tbz2",
+			pkgdir, atom->CATEGORY, pkg->PF);
 	if (pkg_verify_checksums(buf, pkg, atom, qmerge_strict, !quiet) == 0) {
 		pkg_merge(0, atom, pkg);
 		return;
@@ -2021,6 +2035,7 @@ find_binpkg(const char *name)
 		return best_match;
 
 	fp = open_binpkg_index();
+	PF[0] = CATEGORY[0] = '\0';
 
 	while (fgets(buf, sizeof(buf), fp) != NULL) {
 		if (*buf == '\n') {
@@ -2232,6 +2247,8 @@ parse_packages(set *todo)
 					snprintf(Pkg.CATEGORY, sizeof(Pkg.CATEGORY), "%.*s",
 							(int)sizeof(Pkg.CATEGORY) - 1, p);
 				if (strcmp(buf, "CPV") == 0) {
+					if (pkg_atom != NULL)  /* hypothetical Coverity case */
+						atom_implode(pkg_atom);
 					if ((pkg_atom = atom_explode(p)) != NULL) {
 						if (pkg_atom->PR_int)
 							snprintf(Pkg.PF, sizeof(Pkg.PF), "%s-%s-r%i",
